// provides a class to store global statistics (mainly used in webif)

#include "stats.h"
#include <string.h>

// ---------------------------------- statistics class member declarations ---------------------------------
statistics::statistics( void )
{
	started_on[0] = 0;
	
	// -- cardserver client
	strcpy(cs_last_connection_time, "n/a");
	cs_reconnections = 0;
	
	cs_successful_ecms = 0;
	cs_ecm_failures = 0;
	
	cs_successful_emm_g = 0;
	cs_emm_g_failures = 0;

	cs_successful_emm_s = 0;
	cs_emm_s_failures = 0;

	cs_successful_emm_u = 0;
	cs_emm_u_failures = 0;
	
	//-- webif console
	webif_console_line_root = 0;
	
	webif_console_line_buffer_length = 0;
	webif_console_line_buffer_length_max = 80;
	
	// -- nagra2
	memset(n2_ecm_rsa_decryption_key, 0, 64);
	memset(n2_ecm_idea_decryption_key, 0, 16);
	
	n2_mecm_select = 0;
	memset(n2_mecm_idea_decrypt_key, 0, 16);
	memset(n2_mecm_encrypted, 0, 256);
	memset(n2_mecm_decrypted_before_execution, 0, 256);
	memset(n2_mecm_hw_before_execution, 0, 128);
	memset(n2_mecm_decrypted_after_execution, 0, 256);
	memset(n2_mecm_hw_after_execution, 0, 128);
	n2_mecm_execution_time = 0;
	n2_mecm_instruction_count = 0;
	*n2_mecm_execution_summary = 0;
	*n2_mecm_execution_status = 0;
	
	n2_emm_is_valid = false;
	memset(n2_emm_before_b1_nano_execution, 0, 96);
	memset(n2_emm_rsa_decryption_key, 0, 96);
	memset(n2_emm_idea_decryption_key, 0, 24);
	memset(n2_emm_b1_nano, 0, 96);
	n2_emm_b1_nano_length = 0;
	n2_emm_b1_nano_execution_time = 0;
	n2_emm_b1_nano_instruction_count = 0;
	*n2_emm_b1_nano_execution_summary = 0;
	*n2_emm_b1_nano_execution_status = 0;
	*n2_emm_parsing_status = 0;
	memset(n2_emm_after_b1_nano_execution, 0, 96);
}

statistics::~statistics( void )
{
	
}

void statistics::cs_connected( void )
{
	strcpy(cs_last_connection_time, get_time());
	cs_reconnections++;
}

void statistics::webif_console_add_line(char *line)
{
	if( webif_console_line_root )
	{
		//add new line, linking it correctly to the list as the last element
		webif_console_line *current_line = 
			new webif_console_line(line, webif_console_line_root->previous, 0);
		
		webif_console_line_root->previous->next = current_line;
		webif_console_line_root->previous = current_line;
		
		//delete the first element in the list if the line limit was exceeded
		if( webif_console_line_buffer_length == webif_console_line_buffer_length_max )
		{
			webif_console_line *temp = webif_console_line_root;
			
			webif_console_line_root = webif_console_line_root->next;
			webif_console_line_root->previous = temp->previous;
			delete temp;
		}
		else webif_console_line_buffer_length++;
	}
	else
	{
		webif_console_line_root = new webif_console_line(line, 0, 0);
		webif_console_line_root->previous = webif_console_line_root;
		webif_console_line_buffer_length++;
	}
}

void statistics::webif_console_add_text(char *text)
{
	char *start, *end;
	
	start = text;
	do
	{
		end = strstr(start, "\n");
		if( end )
		{
			*end = 0;
			webif_console_add_line( start );
			*end = '\n';
			start = end + 1;
		}
		else
		{
			if(strlen(start) > 0 ) webif_console_add_line( start );
			break;
		}
			
	}
	while( true );
}

// ----------------------------- webif_console_line class member declarations ----------------------------
webif_console_line::webif_console_line( char *init_line, webif_console_line *init_previous, webif_console_line *init_next )
{
	line = new char[strlen(init_line) + 1];
	strcpy(line, init_line);
	
	previous = init_previous;
	next = init_next;
}

webif_console_line::~webif_console_line( void )
{
	delete line;
}
